@@ -1076,6 +1076,21 @@ simplejump_p (insn)
 	  && GET_CODE (SET_DEST (PATTERN (insn))) == PC
 	  && GET_CODE (SET_SRC (PATTERN (insn))) == LABEL_REF);
 }
+/* Return 1 if INSN is an tablejump.  */
+
+int
+tablejump_p (insn)
+     rtx insn;
+{
+  rtx table;
+  return (GET_CODE (insn) == JUMP_INSN
+	  && JUMP_LABEL (insn)
+	  && NEXT_INSN (JUMP_LABEL (insn))
+	  && (table = next_active_insn (JUMP_LABEL (insn)))
+	  && GET_CODE (table) == JUMP_INSN
+	  && (GET_CODE (PATTERN (table)) == ADDR_VEC
+	      || GET_CODE (PATTERN (table)) == ADDR_DIFF_VEC));
+}
 
 /* Return nonzero if INSN is a (possibly) conditional jump
    and nothing more.
@@ -2428,3 +2443,15 @@ true_regnum (x)
     }
   return -1;
 }
+
+/* Return regno of the register REG and handle subregs too.  */
+unsigned int
+reg_or_subregno (reg)
+     rtx reg;
+{
+  if (REG_P (reg))
+    return REGNO (reg);
+  if (GET_CODE (reg) == SUBREG)
+    return REGNO (SUBREG_REG (reg));
+  abort ();
+}
